searchNodes=[{"doc":"This module can be used to start and maintain a pool of Consumer workers. Example usage rabbit_mq allows you to design consistent, SDK-like Consumers. ℹ️ The following example assumes that the &quot;customer/customer.updated&quot; queue already exists. First, define your Consumer(s): defmodule RabbitSample.CustomerCreatedConsumer do use RabbitMQ.Consumer, queue: &quot;customer/customer.created&quot;, worker_count: 2, prefetch_count: 3 require Logger def consume(payload, meta, channel) do Logger.info(&quot;Customer \#{payload} created.&quot;) ack(channel, meta.delivery_tag) end end defmodule RabbitSample.CustomerUpdatedConsumer do use RabbitMQ.Consumer, queue: &quot;customer/customer.updated&quot;, worker_count: 2, prefetch_count: 6 require Logger def consume(payload, meta, channel) do Logger.info(&quot;Customer updated. Data: \#{payload}.&quot;) ack(channel, meta.delivery_tag) end end Then, start as normal under your existing supervision tree: children = [ RabbitSample.Topology, RabbitSample.CustomerProducer, RabbitSample.CustomerCreatedConsumer, RabbitSample.CustomerUpdatedConsumer ] opts = [strategy: :one_for_one, name: RabbitSample.Supervisor] Supervisor.start_link(children, opts) As messages are published onto the &quot;customer/customer.created&quot;, or &quot;customer/customer.updated&quot; queues, the corresponding consume/3 will be invoked. ⚠️ Please note that automatic message acknowledgement is disabled in rabbit_mq, therefore it&#39;s your responsibility to ensure messages are ack&#39;d or nack&#39;d. ℹ️ Please consult the Consumer Acknowledgement Modes and Data Safety Considerations for more details. Configuration The following options can be used with RabbitMQ.Consumer; :prefetch_count; limits the number of unacknowledged messages on a channel. Please consult the Consumer Prefetch section for more details. Defaults to 10. :queue; the name of the queue from which the Consumer should start consuming. For exclusive queues, please see the Exclusive Queues section further below. Required. :worker_count; number of workers to be spawned. Cannot be greater than :max_channels_per_connection set in config. Defaults to 3. When you use RabbitMQ.Consumer, a few things happen; The module turns into a GenServer. The server starts and supervises the desired number of workers. consume/3 is passed as a callback to each worker when start_link/1 is called. If an exclusive queue is requested, it will be declared and bound to the Consumer. Each worker starts consuming from the queue provided, calls consume/3 for each message consumed. ack/2, nack/2, and reject/2 become automatically available. consume/3 needs to be defined with the following signature; @type payload :: String.t() @type meta :: map() @type channel :: AMQP.Channel.t() @type result :: :ok | {:error, :retry} | {:error, term()} consume(payload(), meta(), channel()) :: result() Exclusive queues If you want to consume from an exclusive queue, simply use one of the following tuples in the configuration; {exchange, routing_key} {exchange, routing_key, opts} {exchange, routing_key, queue_name} {exchange, routing_key, queue_name, opts} ℹ️ At a minimum, the exchange and the routing_key are required. defmodule CustomerConsumer do use RabbitMQ.Consumer, queue: {&quot;customer&quot;, &quot;customer.*&quot;}, worker_count: 3 # Define `consume/3` as normal. end This will ensure that the queue is declared and correctly bound before the Consumer workers start consuming messages off it.","ref":"RabbitMQ.Consumer.html","title":"RabbitMQ.Consumer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Consumer.html#child_spec/1","title":"RabbitMQ.Consumer.child_spec/1","type":"function"},{"doc":"Starts this module as a process via GenServer.start_link/3. Only used by the module&#39;s child_spec.","ref":"RabbitMQ.Consumer.html#start_link/2","title":"RabbitMQ.Consumer.start_link/2","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Consumer.html#terminate/2","title":"RabbitMQ.Consumer.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Consumer server. :connection; holds the dedicated AMQP.Connection. :workers; the children started under the server. The server acts as a Supervisor.","ref":"RabbitMQ.Consumer.State.html","title":"RabbitMQ.Consumer.State","type":"module"},{"doc":"The single Consumer worker used to consume messages from a queue. Calls consume/3 defined in the parent module for each message consumed.","ref":"RabbitMQ.Consumer.Worker.html","title":"RabbitMQ.Consumer.Worker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Consumer.Worker.html#child_spec/1","title":"RabbitMQ.Consumer.Worker.child_spec/1","type":"function"},{"doc":"","ref":"RabbitMQ.Consumer.Worker.html#start_link/1","title":"RabbitMQ.Consumer.Worker.start_link/1","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Consumer.Worker.html#terminate/2","title":"RabbitMQ.Consumer.Worker.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Consumer.Worker server. :channel; holds the dedicated AMQP.Channel. :consume_cb; the callback invoked for each message consumed. :consumer_tag; the unique consumer identifier.","ref":"RabbitMQ.Consumer.Worker.State.html","title":"RabbitMQ.Consumer.Worker.State","type":"module"},{"doc":"This module can be used to start and maintain a pool of Producer workers. Example usage rabbit_mq allows you to design consistent, SDK-like Producers. ℹ️ The following example assumes that the &quot;customer&quot; exchange already exists. First, define your (ideally domain-specific) Producer: defmodule RabbitSample.CustomerProducer do @moduledoc &quot;&quot;&quot; Publishes pre-configured events onto the &quot;customer&quot; exchange. &quot;&quot;&quot; use RabbitMQ.Producer, exchange: &quot;customer&quot;, worker_count: 3 @doc &quot;&quot;&quot; Publishes an event routed via &quot;customer.created&quot;. &quot;&quot;&quot; def customer_created(customer_id) when is_binary(customer_id) do opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_id: customer_id}) publish(payload, &quot;customer.created&quot;, opts) end @doc &quot;&quot;&quot; Publishes an event routed via &quot;customer.updated&quot;. &quot;&quot;&quot; def customer_updated(updated_customer) when is_map(updated_customer) do opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_data: updated_customer}) publish(payload, &quot;customer.updated&quot;, opts) end end Then, start as normal under your existing supervision tree: children = [ RabbitSample.Topology, RabbitSample.CustomerProducer, RabbitSample.CustomerCreatedConsumer, RabbitSample.CustomerUpdatedConsumer ] opts = [strategy: :one_for_one, name: RabbitSample.Supervisor] Supervisor.start_link(children, opts) Finally, call the exposed methods from your application: RabbitSample.CustomerProducer.customer_created(customer_id) RabbitSample.CustomerProducer.customer_updated(updated_customer) ⚠️ Please note that all Producer workers implement &quot;reliable publishing&quot;. Each Producer worker handles its publisher confirms asynchronously, striking a delicate balance between performance and reliability. To understand why this is important, please refer to the reliable publishing implementation guide. ℹ️ In the unlikely event of an unexpected Publisher nack, your server will be notified via the on_unexpected_nack/2 callback, letting you handle such exceptions in any way you see fit. Configuration The following options can be used with RabbitMQ.Producer; :confirm_type; publisher acknowledgement mode. Only :async is supported for now. Please consult the Publisher Confirms section for more details. Defaults to :async. :exchange; the name of the exchange onto which the producer workers will publish. Required. :worker_count; number of workers to be spawned. Cannot be greater than :max_channels_per_connection set in config. Defaults to 3. When you use RabbitMQ.Consumer, a few things happen; The module turns into a GenServer. The server starts and supervises the desired number of workers. publish/3 becomes available in your module. publish/3 is a private function with the following signature; @type payload :: String.t() @type routing_key :: String.t() @type opts :: keyword() @type result :: :ok | {:error, :correlation_id_missing} publish(payload(), routing_key(), opts()) :: result() ⚠️ Please note that correlation_id is always required and failing to provide it will result in an exception. ℹ️ To see which options can be passed as opts to publish/3, visit https://hexdocs.pm/amqp/AMQP.Basic.html#publish/5.","ref":"RabbitMQ.Producer.html","title":"RabbitMQ.Producer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Producer.html#child_spec/1","title":"RabbitMQ.Producer.child_spec/1","type":"function"},{"doc":"Starts this module as a process via GenServer.start_link/3. Only used by the module&#39;s child_spec.","ref":"RabbitMQ.Producer.html#start_link/2","title":"RabbitMQ.Producer.start_link/2","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Producer.html#terminate/2","title":"RabbitMQ.Producer.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Producer server. :connection; holds the dedicated AMQP.Connection. :workers; the children started under the server. The server acts as a Supervisor. Enables round-robin command dispatch. :worker_count; a simple worker count tracker. Used in round-robin command dispatch logic. :worker_offset; a simple tracker to determine which worker to call next. Used in round-robin command dispatch logic.","ref":"RabbitMQ.Producer.State.html","title":"RabbitMQ.Producer.State","type":"module"},{"doc":"The single Producer worker used to publish messages onto an exchange.","ref":"RabbitMQ.Producer.Worker.html","title":"RabbitMQ.Producer.Worker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Producer.Worker.html#child_spec/1","title":"RabbitMQ.Producer.Worker.child_spec/1","type":"function"},{"doc":"Starts this module as a process via GenServer.start_link/2. Only used by the parent module which acts as a Supervisor.","ref":"RabbitMQ.Producer.Worker.html#start_link/1","title":"RabbitMQ.Producer.Worker.start_link/1","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Producer.Worker.html#terminate/2","title":"RabbitMQ.Producer.Worker.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Producer.Worker server. :channel; holds the dedicated AMQP.Channel. :outstanding_confirms; holds the reference to the protected ets table used to track outstanding Publisher ack or nack confirms.","ref":"RabbitMQ.Producer.Worker.State.html","title":"RabbitMQ.Producer.Worker.State","type":"module"},{"doc":"A convenience module that can be used to establish the (RabbitMQ) routing topology. First, create a module that uses RabbitMQ.Topology to define exchanges and their corresponding bindings as shown below. ⚠️ Please note that exclusive queues cannot be configured here. You may need to consult the RabbitMQ.Consumer module for details on how exclusive queues can be set up and used. defmodule RabbitSample.Topology do use RabbitMQ.Topology, exchanges: [ {&quot;customer&quot;, :topic, [ {&quot;customer.created&quot;, &quot;customer/customer.created&quot;, durable: true}, {&quot;customer.updated&quot;, &quot;customer/customer.updated&quot;, durable: true} ], durable: true} ] end Then, simply add this module to your supervision tree, before any Consumers or Producers that rely on the exchanges configured within it start. ⚠️ Please note that the Topology module will terminate gracefully as soon as the desired routing is configured. children = [ RabbitSample.Topology, RabbitSample.CustomerProducer, RabbitSample.CustomerCreatedConsumer, RabbitSample.CustomerUpdatedConsumer ] opts = [strategy: :one_for_one, name: RabbitSample.Supervisor] Supervisor.start_link(children, opts)","ref":"RabbitMQ.Topology.html","title":"RabbitMQ.Topology","type":"module"},{"doc":"The macro to use this module. Available options: exchanges: [ { # Exchange name &quot;customer&quot;, # Exchange type, only topic is supported at the moment :topic, # List of bindings [ { # Routing/binding key &quot;#&quot;, # Queue name &quot;customer/#&quot;, # Queue opts (optional) durable: true } ], # Exchange opts durable: true } ]","ref":"RabbitMQ.Topology.html#__using__/1","title":"RabbitMQ.Topology.__using__/1","type":"macro"},{"doc":"MIT License Copyright (c) 2020 Slavomir Vojacek Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"Overview rabbit_mq builds on amqp to deliver modules that are pre-configured with sensible defaults and follow design principles that improve and delicately balance both performance and reliability. The following modules are provided; RabbitMQ.Topology RabbitMQ.Consumer RabbitMQ.Producer","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"⚠️ The following examples assume you&#39;ve already set up your (RabbitMQ) routing topology as shown below. ℹ️ Consult the RabbitMQ.Topology module to learn how to quickly establish desired routing topology. source_namesource_kinddestination_namedestination_kindrouting_keyarguments customerexchangecustomer/customer.createdqueuecustomer.created[] customerexchangecustomer/customer.updatedqueuecustomer.updated[] As seen in the RabbitMQ Management dashboard: First, ensure you point to a valid amqp_url by configuring :rabbit_mq in your config.exs. config :rabbit_mq, :amqp_url, &quot;amqp://guest:guest@localhost:5672&quot; ℹ️ For advanced configuration options, consult the Configuration section. Producers Let&#39;s define our CustomerProducer first. We will use this module to publish messages onto the &quot;customer&quot; exchange. defmodule RabbitSample.CustomerProducer do @moduledoc &quot;&quot;&quot; Publishes pre-configured events onto the &quot;customer&quot; exchange. &quot;&quot;&quot; use RabbitMQ.Producer, exchange: &quot;customer&quot;, worker_count: 3 @doc &quot;&quot;&quot; Publishes an event routed via &quot;customer.created&quot;. &quot;&quot;&quot; def customer_created(customer_id) when is_binary(customer_id) do opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_id: customer_id}) publish(payload, &quot;customer.created&quot;, opts) end @doc &quot;&quot;&quot; Publishes an event routed via &quot;customer.updated&quot;. &quot;&quot;&quot; def customer_updated(updated_customer) when is_map(updated_customer) do opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_data: updated_customer}) publish(payload, &quot;customer.updated&quot;, opts) end end ⚠️ Please note that all Producer workers implement &quot;reliable publishing&quot;. Each Producer worker handles its publisher confirms asynchronously, striking a delicate balance between performance and reliability. To understand why this is important, please refer to the reliable publishing implementation guide. ℹ️ In the unlikely event of an unexpected Publisher nack, your server will be notified via the on_unexpected_nack/2 callback, letting you handle such exceptions in any way you see fit. Consumers To consume messages off the respective queues, we will define 2 separate consumers. ⚠️ Please note that automatic message acknowledgement is disabled in rabbit_mq, therefore it&#39;s your responsibility to ensure messages are ack&#39;d or nack&#39;d. ℹ️ Please consult the Consumer Acknowledgement Modes and Data Safety Considerations for more details. defmodule RabbitSample.CustomerCreatedConsumer do use RabbitMQ.Consumer, queue: &quot;customer/customer.created&quot;, worker_count: 2, prefetch_count: 3 require Logger def consume(payload, meta, channel) do Logger.info(&quot;Customer \#{payload} created.&quot;) ack(channel, meta.delivery_tag) end end defmodule RabbitSample.CustomerUpdatedConsumer do use RabbitMQ.Consumer, queue: &quot;customer/customer.updated&quot;, worker_count: 2, prefetch_count: 6 require Logger def consume(payload, meta, channel) do Logger.info(&quot;Customer updated. Data: \#{payload}.&quot;) ack(channel, meta.delivery_tag) end end Use under supervision tree And finally, we will start our application. ℹ️ To run RabbitMQ locally, see our docker-compose.yaml for a sample Docker Compose set up. defmodule RabbitSample.Application do use Application def start(_type, _args) do children = [ RabbitSample.CustomerProducer, RabbitSample.CustomerCreatedConsumer, RabbitSample.CustomerUpdatedConsumer ] opts = [strategy: :one_for_one, name: RabbitSample.Supervisor] Supervisor.start_link(children, opts) end end Using iex; iex -S mix The resulting application topology should look like this: Upon closer inspection using the RabbitMQ Management dashboard, we see that: a) each of our modules maintains its dedicated connection; and b) each of our modules&#39; workers maintains its dedicated channel under the respective connection. ℹ️ Detailed view of how individual workers have set up their channels. Note that the different prefetch counts correspond to the different configuration we provided in our Consumers, and that the Producer&#39;s 3 worker channels operate in Confirm mode. Produce and Consume messages iex(1)&gt; RabbitSample.CustomerProducer.customer_created(UUID.uuid4()) {:ok, 1} 14:07:22.058 application=rabbit_mq domain=elixir file=lib/rabbit_mq/producer/producer_worker.ex function=handle_info/2 line=84 mfa=RabbitMQ.Producer.Worker.handle_info/2 module=RabbitMQ.Producer.Worker pid=&lt;0.317.0&gt; [debug] Received ACK of 1. 14:07:22.058 application=rabbit_sample domain=elixir file=lib/rabbit_sample/customer_created_consumer.ex function=consume/3 line=7 mfa=RabbitSample.CustomerCreatedConsumer.consume/3 module=RabbitSample.CustomerCreatedConsumer pid=&lt;0.378.0&gt; [info] Customer {&quot;customer_id&quot;:&quot;e79eae21-b8a4-4907-9795-5aa633a9d3df&quot;,&quot;v&quot;:&quot;1.0.0&quot;} created. iex(2)&gt; RabbitSample.CustomerProducer.customer_updated(%{id: UUID.uuid4()}) {:ok, 1} 14:07:39.247 application=rabbit_sample domain=elixir file=lib/rabbit_sample/customer_updated_consumer.ex function=consume/3 line=7 mfa=RabbitSample.CustomerUpdatedConsumer.consume/3 module=RabbitSample.CustomerUpdatedConsumer pid=&lt;0.381.0&gt; [info] Customer updated. Data: {&quot;customer_data&quot;:{&quot;id&quot;:&quot;eee24a7e-ce7b-4eec-8d5f-47ee98f1ca6f&quot;},&quot;v&quot;:&quot;1.0.0&quot;}. 14:07:39.247 application=rabbit_mq domain=elixir file=lib/rabbit_mq/producer/producer_worker.ex function=handle_info/2 line=84 mfa=RabbitMQ.Producer.Worker.handle_info/2 module=RabbitMQ.Producer.Worker pid=&lt;0.323.0&gt; [debug] Received ACK of 1. iex(3)&gt;","ref":"overview.html#sample-usage","title":"Overview - Sample usage","type":"extras"},{"doc":"The following options can be configured. config :rabbit_mq, amqp_url: &quot;amqp://guest:guest@localhost:5672&quot;, heartbeat_interval_sec: 60, reconnect_interval_ms: 2500, max_channels_per_connection: 16 amqp_url; required, the broker URL. heartbeat_interval_sec; defines after what period of time the peer TCP connection should be considered unreachable. Defaults to 30. reconnect_interval_ms; the interval before another attempt to re-connect to the broker should occur. Defaults to 2500. max_channels_per_connection; maximum number of channels per connection. Also determines the maximum number of workers per Producer/Consumer module. Defaults to 8. ⚠️ Please consult the Channels Resource Usage guide to understand how to best configure :max_channels_per_connection. ⚠️ Please consult the Detecting Dead TCP Connections with Heartbeats and TCP Keepalives guide to understand how to best configure :heartbeat_interval_sec. Excessive logging See original section in amqp docs. Add the following configuration. config :logger, handle_otp_reports: false Lager conflicts with Elixir logger Lager is used by rabbit_common and is not Elixir&#39;s best friend yet. You need a workaround. ⚠️ In mix.exs, you have to load :lager before :logger. extra_applications: [:lager, :logger]","ref":"overview.html#configuration","title":"Overview - Configuration","type":"extras"},{"doc":"The library itself has been rigorously tested, so you should ideally only need to test whether you&#39;ve configured your modules correctly. Additionally, you should test any side-effects driven by your Producers or Consumers. Producers Here is a few ideas on how you can test your Producers. ⚠️ The below snippet assumes your application starts the CustomerProducer module as shown in earlier examples. defmodule RabbitSampleTest.CustomerProducer do alias AMQP.{Basic, Channel, Connection, Queue} alias RabbitSample.CustomerProducer use ExUnit.Case @amqp_url Application.get_env(:rabbit_mq, :amqp_url) @exchange &quot;customer&quot; setup_all do assert {:ok, connection} = Connection.open(@amqp_url) assert {:ok, channel} = Channel.open(connection) # Declare an exclusive queue and bind it to the customer exchange. {:ok, %{queue: queue}} = Queue.declare(channel, &quot;&quot;, exclusive: true) :ok = Queue.bind(channel, queue, @exchange, routing_key: &quot;#&quot;) # Clean up after all tests have ran. on_exit(fn -&gt; # This queue would have been deleted automatically when the connection # gets closed, however we prefer to be explicit. Also, we ensure there # are no messages left hanging in the queue. assert {:ok, %{message_count: 0}} = Queue.delete(channel, queue) assert :ok = Channel.close(channel) assert :ok = Connection.close(connection) end) [channel: channel, queue: queue] end setup %{channel: channel, queue: queue} do # Each test will be notified when a message is consumed. assert {:ok, consumer_tag} = Basic.consume(channel, queue) # This will always be the first message received by the process. assert_receive({:basic_consume_ok, %{consumer_tag: ^consumer_tag}}) on_exit(fn -&gt; # Ensure there are no messages in the queue as the next test is about to start. assert true = Queue.empty?(channel, queue) end) [ channel: channel, consumer_tag: consumer_tag ] end describe &quot;\#{__MODULE__}&quot; do test &quot;defines correctly configured child specification&quot;, %{ channel: channel, consumer_tag: consumer_tag } do assert %{ id: CustomerProducer, restart: :permanent, shutdown: :brutal_kill, start: {RabbitMQ.Producer, :start_link, [ %{confirm_type: :async, exchange: @exchange, worker_count: 3}, [name: CustomerProducer] ]}, type: :supervisor } = CustomerProducer.child_spec([]) Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end test &quot;customer_created/1 publishes correctly configured events&quot;, %{ channel: channel, consumer_tag: consumer_tag } do customer_id = UUID.uuid4() expected_payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_id: customer_id}) assert {:ok, _seq_no} = CustomerProducer.customer_created(customer_id) assert_receive( {:basic_deliver, ^expected_payload, %{ consumer_tag: ^consumer_tag, content_type: &quot;application/json&quot;, correlation_id: correlation_id, delivery_tag: delivery_tag, routing_key: &quot;customer.created&quot; }} ) # Ensure correlation_id is a valid UUID. assert {:ok, _} = UUID.info(correlation_id) # Acknowledge that the message has been received. Basic.ack(channel, delivery_tag) # Stop consuming. Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end test &quot;customer_updated/1 publishes correctly configured events&quot;, %{ channel: channel, consumer_tag: consumer_tag } do customer_data = %{id: UUID.uuid4()} expected_payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_data: customer_data}) assert {:ok, _seq_no} = CustomerProducer.customer_updated(customer_data) assert_receive( {:basic_deliver, ^expected_payload, %{ consumer_tag: ^consumer_tag, content_type: &quot;application/json&quot;, correlation_id: correlation_id, delivery_tag: delivery_tag, routing_key: &quot;customer.updated&quot; }} ) # Ensure correlation_id is a valid UUID. assert {:ok, _} = UUID.info(correlation_id) # Acknowledge that the message has been received. Basic.ack(channel, delivery_tag) # Stop consuming. Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end end end Consumers Here is a few ideas on how you can test your Consumers. ⚠️ The below snippet assumes your application starts the CustomerCreatedConsumer module as shown in earlier examples. defmodule RabbitSampleTest.CustomerCreatedConsumer do alias AMQP.{Basic, Channel, Connection, Queue} alias RabbitSample.CustomerCreatedConsumer import ExUnit.CaptureLog use ExUnit.Case @amqp_url Application.get_env(:rabbit_mq, :amqp_url) @exchange &quot;customer&quot; @queue &quot;\#{@exchange}/customer.created&quot; setup_all do assert {:ok, connection} = Connection.open(@amqp_url) assert {:ok, channel} = Channel.open(connection) # Declare an exclusive queue and bind it to the customer exchange. {:ok, %{queue: queue}} = Queue.declare(channel, &quot;&quot;, exclusive: true) :ok = Queue.bind(channel, queue, @exchange, routing_key: &quot;#&quot;) # Clean up after all tests have ran. on_exit(fn -&gt; # This queue would have been deleted automatically when the connection # gets closed, however we prefer to be explicit. Also, we ensure there # are no messages left hanging in the queue. assert {:ok, %{message_count: 0}} = Queue.delete(channel, queue) assert :ok = Channel.close(channel) assert :ok = Connection.close(connection) end) [channel: channel, queue: queue] end setup %{channel: channel, queue: queue} do # Each test will be notified when a message is consumed. assert {:ok, consumer_tag} = Basic.consume(channel, queue) # This will always be the first message received by the process. assert_receive({:basic_consume_ok, %{consumer_tag: ^consumer_tag}}) on_exit(fn -&gt; # Ensure there are no messages in the queue as the next test is about to start. assert true = Queue.empty?(channel, queue) end) [ channel: channel, consumer_tag: consumer_tag ] end describe &quot;\#{__MODULE__}&quot; do test &quot;defines correctly configured child specification&quot;, %{ channel: channel, consumer_tag: consumer_tag } do assert %{ id: CustomerCreatedConsumer, restart: :permanent, shutdown: :brutal_kill, start: {RabbitMQ.Consumer, :start_link, [ %{consume_cb: _, prefetch_count: 3, queue: @queue, worker_count: 2}, [name: CustomerCreatedConsumer] ]}, type: :supervisor } = CustomerCreatedConsumer.child_spec([]) # Stop consuming. Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end test &quot;consume/3 logs a message&quot;, %{ channel: channel, consumer_tag: consumer_tag } do correlation_id = UUID.uuid4() payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_id: UUID.uuid4()}) Basic.publish(channel, @exchange, &quot;customer.created&quot;, payload, correlation_id: correlation_id ) assert_receive( {:basic_deliver, payload, %{ consumer_tag: ^consumer_tag, correlation_id: ^correlation_id, routing_key: &quot;customer.created&quot; } = meta} ) assert capture_log(fn -&gt; CustomerCreatedConsumer.consume(payload, meta, channel) end) =~ &quot;Customer \#{payload} created&quot; # Stop consuming. Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end end end ⚠️ The below snippet assumes your application starts the CustomerUpdatedConsumer module as shown in earlier examples. defmodule RabbitSampleTest.CustomerUpdatedConsumer do alias AMQP.{Basic, Channel, Connection, Queue} alias RabbitSample.CustomerUpdatedConsumer import ExUnit.CaptureLog use ExUnit.Case @amqp_url Application.get_env(:rabbit_mq, :amqp_url) @exchange &quot;customer&quot; @queue &quot;\#{@exchange}/customer.updated&quot; setup_all do assert {:ok, connection} = Connection.open(@amqp_url) assert {:ok, channel} = Channel.open(connection) # Declare an exclusive queue and bind it to the customer exchange. {:ok, %{queue: queue}} = Queue.declare(channel, &quot;&quot;, exclusive: true) :ok = Queue.bind(channel, queue, @exchange, routing_key: &quot;#&quot;) # Clean up after all tests have ran. on_exit(fn -&gt; # This queue would have been deleted automatically when the connection # gets closed, however we prefer to be explicit. Also, we ensure there # are no messages left hanging in the queue. assert {:ok, %{message_count: 0}} = Queue.delete(channel, queue) assert :ok = Channel.close(channel) assert :ok = Connection.close(connection) end) [channel: channel, queue: queue] end setup %{channel: channel, queue: queue} do # Each test will be notified when a message is consumed. assert {:ok, consumer_tag} = Basic.consume(channel, queue) # This will always be the first message received by the process. assert_receive({:basic_consume_ok, %{consumer_tag: ^consumer_tag}}) on_exit(fn -&gt; # Ensure there are no messages in the queue as the next test is about to start. assert true = Queue.empty?(channel, queue) end) [ channel: channel, consumer_tag: consumer_tag ] end describe &quot;\#{__MODULE__}&quot; do test &quot;defines correctly configured child specification&quot;, %{ channel: channel, consumer_tag: consumer_tag } do assert %{ id: CustomerUpdatedConsumer, restart: :permanent, shutdown: :brutal_kill, start: {RabbitMQ.Consumer, :start_link, [ %{consume_cb: _, prefetch_count: 6, queue: @queue, worker_count: 2}, [name: CustomerUpdatedConsumer] ]}, type: :supervisor } = CustomerUpdatedConsumer.child_spec([]) # Stop consuming. Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end test &quot;consume/3 logs a message&quot;, %{ channel: channel, consumer_tag: consumer_tag } do correlation_id = UUID.uuid4() customer_data = %{id: UUID.uuid4()} payload = Jason.encode!(%{v: &quot;1.0.0&quot;, customer_data: customer_data}) Basic.publish(channel, @exchange, &quot;customer.updated&quot;, payload, correlation_id: correlation_id ) assert_receive( {:basic_deliver, payload, %{ consumer_tag: ^consumer_tag, correlation_id: ^correlation_id, routing_key: &quot;customer.updated&quot; } = meta} ) assert capture_log(fn -&gt; CustomerUpdatedConsumer.consume(payload, meta, channel) end) =~ &quot;Customer updated. Data: \#{payload}.&quot; # Stop consuming. Basic.cancel(channel, consumer_tag) # This will always be the last message received by the process. assert_receive({:basic_cancel_ok, %{consumer_tag: ^consumer_tag}}) # Ensure no further messages are received. refute_receive(_) end end end","ref":"overview.html#testing","title":"Overview - Testing","type":"extras"},{"doc":"The RabbitMQ modules are pre-configured with sensible defaults and follow design principles that improve and delicately balance both performance and reliability. This has been possible through a) extensive experience of working with Elixir and RabbitMQ in production; and b) meticulous consultation of the below (and more) documents and guides. ⚠️ While most of the heavy-lifting is provided by the library itself, reading through the documents below before running any application in production is thoroughly recommended. Connections Channels Reliability Guide Consumer Acknowledgements and Publisher Confirms Consumer Acknowledgement Modes and Data Safety Considerations Reliable publishing with publisher confirms Consumer Prefetch Production Checklist RabbitMQ Best Practices RabbitMQ Best Practice for High Performance (High Throughput)","ref":"overview.html#balanced-performance-and-reliability","title":"Overview - Balanced performance and reliability","type":"extras"},{"doc":"Topology ℹ️ This section is currently being completed. Thank you for your understanding and patience. Please refer to the RabbitMQ.Topology module for documentation.","ref":"topology.html","title":"Topology","type":"extras"},{"doc":"Consumers ℹ️ This section is currently being completed. Thank you for your understanding and patience. Please refer to the RabbitMQ.Consumer module for documentation.","ref":"consumers.html","title":"Consumers","type":"extras"},{"doc":"Producers ℹ️ This section is currently being completed. Thank you for your understanding and patience. Please refer to the RabbitMQ.Producer module for documentation.","ref":"producers.html","title":"Producers","type":"extras"}]